package com.crio.threads.weatherfetcher;

import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

public class WeatherFetcherCallable implements Callable<String> {
    private String city;
    private double latitude;
    private double longitude;
    private static final String APP_ID = "2054e475";
    private static final String APP_KEY = "5d12d75426a488379dcf5c27b9bef6f7";

    public WeatherFetcherCallable(String city, double latitude, double longitude) {
        this.city = city;
        this.latitude = latitude;
        this.longitude = longitude;
    }

    @Override
    public String call() throws Exception {
        // Simulate API request to OpenWeatherMap
        return fetchWeather(latitude, longitude);
    }

    private String fetchWeather(double lat, double lon) throws Exception {
        String urlString = "http://api.weatherunlocked.com/api/forecast/" + lat + "," + lon
            + "?app_id=" + APP_ID + "&app_key=" + APP_KEY;
        System.out.println(urlString);
        URL url = new URL(urlString);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("GET");

        int responseCode = connection.getResponseCode();
        if (responseCode == HttpURLConnection.HTTP_OK) {
            BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            String inputLine;
            StringBuilder response = new StringBuilder();

            while ((inputLine = in.readLine()) != null) {
                response.append(inputLine);
            }
            in.close();
            return response.toString();
        } else {
            throw new Exception("GET request failed. Response Code: " + responseCode);
        }
    }

    public static void main(String[] args) {
        // Example coordinates for London
        double lat = 51.5074;
        double lon = -0.1278;
        ExecutorService executor = Executors.newFixedThreadPool(1);
        Callable<String> task = new WeatherFetcherCallable("London", lat, lon);
        Future<String> future = executor.submit(task);

        try {
            // Check if the task is done
            if (future.isDone()) {
                System.out.println("Task is already completed");
            } else {
                // Wait and retrieve the result of the API request
                String weatherData = future.get(10, TimeUnit.SECONDS); // Timeout after 10 seconds
                System.out.println("Weather data for London: " + weatherData);
            }
        } catch (InterruptedException | ExecutionException e) {
            e.printStackTrace();
        } catch (TimeoutException e) {
            System.out.println("The request timed out.");
            boolean cancelled = future.cancel(true);
            System.out.println("Task cancelled: " + cancelled);
        } finally {
            executor.shutdown();
        }
    }
}