package com.crio.threads.weatherfetcher;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

public class WeatherFetcherRunnable implements Runnable {
    private String city;
    private double latitude;
    private double longitude;
    private static final String APP_ID = "2054e475";
    private static final String APP_KEY = "5d12d75426a488379dcf5c27b9bef6f7";

    public WeatherFetcherRunnable(String city, double latitude, double longitude) {
        this.city = city;
        this.latitude = latitude;
        this.longitude = longitude;
    }

    @Override
    public void run() {
        try {
            String response = fetchWeather(latitude, longitude);
            System.out.println("Weather data for " + city + ": " + response);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private String fetchWeather(double lat, double lon) throws Exception {
        String urlString = "http://api.weatherunlocked.com/api/forecast/" + lat + "," + lon
            + "?app_id=" + APP_ID + "&app_key=" + APP_KEY;
        System.out.println(urlString);
        URL url = new URL(urlString);
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("GET");

        int responseCode = connection.getResponseCode();
        if (responseCode == HttpURLConnection.HTTP_OK) {
            BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            String inputLine;
            StringBuilder response = new StringBuilder();

            while ((inputLine = in.readLine()) != null) {
                response.append(inputLine);
            }
            in.close();
            return response.toString();
        } else {
            throw new Exception("GET request failed. Response Code: " + responseCode);
        }
    }

    public static void main(String[] args) {
        // Example coordinates for London
        double lat = 51.5074;
        double lon = -0.1278;
        ExecutorService executor = Executors.newFixedThreadPool(1);
        executor.execute(new WeatherFetcherRunnable("London", lat, lon));
        executor.shutdown();
    }
}

